@@ -124,14 +124,17 @@ def collectSymbols(filename, path=""):
                 f = dirname + "/" + f
             s, i = collectSymbols(f, path + "/" + uuid)
             symbols += s
-            instances += i
+            # The instances shouldn't appear in subsheets, however, in cases
+            # when the sheet used to be a top-level sheet, they are preserved by
+            # KiCAD. So we intentionally ignore them.
+            #
+            # instances += i
             continue
         if isSymbolInstances(item):
             for p in item.items:
                 if isPath(p):
                     instances.append(extractSymbolInstance(p))
             continue
-
     return symbols, instances
 
 
