@@ -1,4 +1,7 @@
 import csv
+from dataclasses import dataclass
+import re
+from typing import OrderedDict
 from pcbnewTransition import pcbnew, isV6
 from math import sin, cos, radians
 from kikit.common import *
@@ -40,7 +43,12 @@ def getReference(component):
 def ensurePassingDrc(board):
     if not isV6():
         return # v5 cannot check DRC
-    failed = drc.runImpl(board, True, False, ReportLevel.error, lambda x: print(x))
+    failed = drc.runImpl(board,
+        useMm=True,
+        ignoreExcluded=True,
+        strict=False,
+        level=ReportLevel.error,
+        yieldViolation=lambda x: print(x))
     if failed:
         print("DRC failed. See report above. No files produced")
         sys.exit(1)
@@ -54,6 +62,15 @@ def hasNonSMDPins(footprint):
 class FormatError(Exception):
     pass
 
+@dataclass
+class CorrectionPattern:
+    """Single correction pattern to match a component against."""
+    footprint: re.Pattern
+    part_id: re.Pattern
+    x_correction: float
+    y_correction: float
+    rotation: float
+
 def layerToSide(layer):
     if layer == pcbnew.F_Cu:
         return "T"
@@ -91,8 +108,54 @@ def excludeFromPos(footprint):
     else:
         return footprint.GetAttributes() & MODULE_ATTR_T.MOD_VIRTUAL
 
+def readCorrectionPatterns(filename):
+    """
+    Read footprint correction pattern file.
+
+    The file should be a CSV file with the following fields:
+    - Regexp to match to the footprint
+    - Regexp to match to the part id (ignored at the moment)
+    - X correction
+    - Y correction
+    - Rotation
+    """
+    corrections = OrderedDict()
+    correctionPatterns = []
+    with open(filename) as csvfile:
+        sample = csvfile.read(1024)
+        dialect = csv.Sniffer().sniff(sample)
+        has_header = csv.Sniffer().has_header(sample)
+        csvfile.seek(0)
+        reader = csv.reader(csvfile, dialect)
+        first = True
+        for row in reader:
+            if has_header and first:
+                first = False
+                continue
+            correctionPatterns.append(
+                CorrectionPattern(
+                    re.compile(row[0]),
+                    re.compile(row[1]),
+                    float(row[2]),
+                    float(row[3]),
+                    float(row[4]),
+                )
+            )
+    return correctionPatterns
+
+def applyCorrectionPattern(correctionPatterns, footprint):
+    # FIXME: part ID is currently ignored
+    # GetUniStringLibId returns the full footprint name including the
+    # library in the form of "Resistor_SMD:R_0402_1005Metric"
+    footprintName = str(footprint.GetFPID().GetUniStringLibId())
+    for corpat in correctionPatterns:
+        if corpat.footprint.match(footprintName):
+            return (corpat.x_correction, corpat.y_correction, corpat.rotation)
+    return (0, 0, 0)
+
 def collectPosData(board, correctionFields, posFilter=lambda x : True,
-                   footprintX=defaultFootprintX, footprintY=defaultFootprintY, bom=None):
+                   footprintX=defaultFootprintX, footprintY=defaultFootprintY, bom=None,
+                   correctionFile=None):
     """
     Extract position data of the footprints.
 
@@ -106,6 +169,11 @@ def collectPosData(board, correctionFields, posFilter=lambda x : True,
         bom = {}
     else:
         bom = { getReference(comp): comp for comp in bom }
+
+    correctionPatterns = []
+    if correctionFile is not None:
+        correctionPatterns = readCorrectionPatterns(correctionFile)
+
     footprints = []
     placeOffset = board.GetDesignSettings().GetAuxOrigin()
     for footprint in board.GetFootprints():
@@ -122,7 +190,9 @@ def collectPosData(board, correctionFields, posFilter=lambda x : True,
             if field is not None:
                 break
         if field is None or field == "":
-            return 0, 0, 0
+            return applyCorrectionPattern(
+                correctionPatterns,
+                footprint)
         try:
             return parseCompensation(field)
         except FormatError as e:
@@ -138,4 +208,21 @@ def posDataToFile(posData, filename):
         writer = csv.writer(csvfile)
         writer.writerow(["Designator", "Mid X", "Mid Y", "Layer", "Rotation"])
         for line in sorted(posData, key=lambda x: x[0]):
+            line = list(line)
+            for i in [1, 2, 4]:
+                line[i] = f"{line[i]:.2f}" # Most Fab houses expect only 2 decimal digits
             writer.writerow(line)
+
+def isValidSchPath(filename):
+    return os.path.splitext(filename)[1] in [".sch", ".kicad_sch"]
+
+def isValidBoardPath(filename):
+    return os.path.splitext(filename)[1] in [".kicad_pcb"]
+
+def ensureValidSch(filename):
+    if not isValidSchPath(filename):
+        raise RuntimeError(f"The path {filename} is not a valid KiCAD schema file")
+
+def ensureValidBoard(filename):
+    if not isValidBoardPath(filename):
+        raise RuntimeError(f"The path {filename} is not a valid KiCAD PCB file")
