@@ -43,6 +43,8 @@ def collectBom(components, manufacturerFields, partNumberFields,
         reference = getReference(c)
         if reference.startswith("#PWR") or reference.startswith("#FL") or reference in ignore:
             continue
+        if hasattr(c, "in_bom") and not c.in_bom:
+            continue
         manufacturer = None
         for manufacturerName in manufacturerFields:
             manufacturer = getField(c, manufacturerName)
@@ -118,10 +120,11 @@ def bomToCsv(bomData, filename, nBoards, types):
 
 def exportPcbway(board, outputdir, assembly, schematic, ignore,
                  manufacturer, partnumber, description, notes, soldertype,
-                 footprint, corrections, missingerror, nboards, nametemplate, drc):
+                 footprint, corrections, correctionpatterns, missingerror, nboards, nametemplate, drc):
     """
     Prepare fabrication files for PCBWay including their assembly service
     """
+    ensureValidBoard(board)
     loadedBoard = pcbnew.LoadBoard(board)
 
     if drc:
@@ -141,6 +144,10 @@ def exportPcbway(board, outputdir, assembly, schematic, ignore,
         return
     if schematic is None:
         raise RuntimeError("When outputing assembly data, schematic is required")
+
+    ensureValidSch(schematic)
+
+
     components = extractComponents(schematic)
     correctionFields    = [x.strip() for x in corrections.split(",")]
     manufacturerFields  = [x.strip() for x in manufacturer.split(",")]
@@ -164,7 +171,7 @@ def exportPcbway(board, outputdir, assembly, schematic, ignore,
     if missingFields and missingerror:
         sys.exit("There are components with missing ordercode, aborting")
 
-    posData = collectPosData(loadedBoard, correctionFields, bom=components)
+    posData = collectPosData(loadedBoard, correctionFields, bom=components, correctionFile=correctionpatterns)
     posDataToFile(posData, os.path.join(outputdir, nametemplate.format("pos") + ".csv"))
     types = collectSolderTypes(loadedBoard)
     bomToCsv(bom, os.path.join(outputdir, nametemplate.format("bom") + ".csv"), nboards, types)
