@@ -114,16 +114,14 @@ class Template:
             boardDesc["gerbers"] = os.path.join(dirPrefix, boardName + "-gerbers.zip")
             boardDesc["file"] = os.path.join(dirPrefix, boardName + ".kicad_pcb")
 
-            subprocess.check_call([pcbdraw, "--vcuts", "--silent", boardDesc["source"],
+            subprocess.check_call([pcbdraw, "plot", "--vcuts=Cmts.User", "--silent", "--side=front", boardDesc["source"],
                 os.path.join(outputDirectory, boardDesc["front"])])
-            subprocess.check_call([pcbdraw, "--vcuts", "--silent", "--back", boardDesc["source"],
+            subprocess.check_call([pcbdraw, "plot", "--vcuts=Cmts.User", "--silent", "--side=back", boardDesc["source"],
                 os.path.join(outputDirectory, boardDesc["back"])])
 
             tmp = tempfile.mkdtemp()
             export.gerberImpl(boardDesc["source"], tmp)
-            gerbers = [os.path.join(tmp, x) for x in os.listdir(tmp)]
-            subprocess.check_call(["zip", "-j",
-                os.path.join(outputDirectory, boardDesc["gerbers"])] + gerbers)
+            shutil.make_archive(os.path.join(outputDirectory, boardDesc["gerbers"])[:-4], "zip", tmp)
             shutil.rmtree(tmp)
 
             shutil.copy(boardDesc["source"], os.path.join(outputDirectory, boardDesc["file"]))
